@@ -19,7 +19,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import threading
-from .devices import Device, Value, NullValue, TecpelDMM8061, XLS200, KernPCB, BS600
+import devices as devicesModule
 import si
 
 class DeviceManager(object):
@@ -29,22 +29,30 @@ class DeviceManager(object):
     get their most recent measurement values and various other things.
 
     """
-    
-    _validDevices = ["TecpelDMM8061", "XLS200", "KernPCB", "BS600"]
+
     _running = False
     _stopEvent = None
+    _stoppedEvent = None
     _thread = None
 
+    _iterator = None
+
     devices = {}
     configs = {}
     rawValues = {}
 
     def __init__(self):
-        pass
+        self._iterator = self._getIterator()
 
-    def _checkConfig(self, config):
-        assert config["deviceName"] in self._validDevices
-        assert isinstance(config["parent"], str) or isinstance(config["parent"], float)  # FIXME: Check for deviceID object when implemented
+    def _getIterator(self):
+        import random
+        i = random.getrandbits(32)
+        while True:
+            yield i
+            i += 1
+
+    def _getUniqueID(self):
+        return next(self._iterator)
 
     def _updateRawValues(self):
         if self._running:
@@ -86,7 +94,7 @@ class DeviceManager(object):
 
         """
 
-        return deviceName in self._validDevices
+        return deviceName in devicesModule.deviceClassNames
 
     def getValidDevices(self):
         """Returns a list of valid deviceNames.
@@ -95,8 +103,9 @@ class DeviceManager(object):
         :rtype: List of strings
 
         """
-
-        return self._validDevices
+        names = list(devicesModule.deviceClassNames)
+        names.sort()
+        return names
 
     def getAllDeviceIDs(self):
         """Returns a complete list of registered deviceIDs.
@@ -191,28 +200,25 @@ class DeviceManager(object):
         :rtype: :class:`DeviceID`
 
         """
-
-        import time
-
+        
         assert self._running == False # FIXME: use another error
+        assert config["deviceName"] in devicesModule.deviceClassNames
 
-        self._checkConfig(config)
-        id = time.time()
-
-        parentID = config["parent"]
+        id = self._getUniqueID()
 
         config["subDevices"] = {}
 
-        if isinstance(parentID, str):
-            device = eval(config["deviceName"]).openRS232(config["parent"], *config["args"], **config["kwargs"])
+        if isinstance(config["parent"], str):
+            device = eval("devicesModule." + config["deviceName"]).openRS232(config["parent"],
+                *config["args"], **config["kwargs"])
 
-        elif isinstance(parentID, float): # FIXME: Check for deviceID object when implemented
-            assert self.devices[parentID] # FIXME: use another error (e.g. WrongIDError)
+        else:
+            assert self.devices[config["parent"]] # FIXME: use another error (e.g. WrongIDError)
 
-            self.devices[parentID].openDevice(eval(config["deviceName"]),
+            self.devices[config["parent"]].openDevice(eval("devicesModule." + config["deviceName"]),
                 input = config["inputNumber"], *config["args"], **config["kwargs"])
-            device = self.devices[parentID].getDevice(input = config["inputNumber"])
-            self.configs[parentID]["subDevices"][id] = config["inputNumber"]
+            device = self.devices[config["parent"]].getDevice(input = config["inputNumber"])
+            self.configs[config["parent"]]["subDevices"][id] = config["inputNumber"]
 
         self.devices[id] = device
         self.configs[id] = config
@@ -244,7 +250,7 @@ class DeviceManager(object):
             del(self.devices[deviceID])
             del(self.configs[deviceID])
 
-        elif isinstance(config["parent"], float): # FIXME: Check for deviceID object when implemented
+        else:
             parentID = config["parent"]
             self.getDevice(parentID).closeDevice(input=config["inputNumber"])
             del(self.devices[deviceID])
@@ -277,10 +283,10 @@ class DeviceManager(object):
             return self.rawValues[deviceID]
 
         except KeyError:
-            import sys
-            sys.stderr.write("KeyError in getLastRawValue (no value yet?)\n")
+            # import sys
+            # sys.stderr.write("KeyError in getLastRawValue (no value yet?)\n")
             
-            return NullValue()
+            return devicesModule.NullValue()
 
     def getCalibratedLastRawValue(self, deviceID, calibration, unit=None):
         """Return a calibrated value from the getLastRawValue method.
@@ -304,7 +310,7 @@ class DeviceManager(object):
             def resultFunction(x): return (calibration[0] * x + calibration[1])
             linearFunction = resultFunction
 
-        class __Value(Value):
+        class __Value(devicesModule.Value):
             value = None
             unit = None
 
@@ -354,20 +360,24 @@ class DeviceManager(object):
 
     def start(self):
         """Start the DeviceManager."""
-
-        self._stopEvent = threading.Event()
-        self._thread = _GetValuesThread(self._stopEvent, self.devices, self.configs)
-        self._thread.start()
-        self._running = True
+        if self._running == False:
+            self._stopEvent = threading.Event()
+            self._stoppedEvent = threading.Event()
+            self._thread = _GetValuesThread(self._stopEvent, self._stoppedEvent,
+                self.devices, self.configs)
+            self._thread.start()
+            self._running = True
 
     def stop(self):
         """Stop the DeviceManager."""
 
         if self._running == True:
             self._stopEvent.set()
-            self._running = False
+            self._stoppedEvent.wait() # wait until thread finishes
             self._thread = None
             self._stopEvent = None
+            self._stoppedEvent = None
+            self._running = False
 
 
 class _GetValuesThread(threading.Thread):
@@ -375,25 +385,31 @@ class _GetValuesThread(threading.Thread):
     configs = {}
     rawValues = {}
 
-    def __init__(self, stop_event, devices, configs):
+    def __init__(self, stop_event, stopped_event, devices, configs):
         threading.Thread.__init__(self)
         self.daemon = True
         self.stop_event = stop_event
+        self.stopped_event = stopped_event
 
         self.devices = devices
         self.configs = configs
 
         for key in self.devices.iteritems():
-            self.rawValues[key] = NullValue()
+            self.rawValues[key] = devicesModule.NullValue()
 
     def run(self):
         while not self.stop_event.is_set():
             self.updateValue()
 
+        self.stopped_event.set()
+
     def updateValue(self):
         # python3 incompatibility: iteritems
         try:
             for key, config in self.configs.iteritems():
+                if self.stop_event.is_set():
+                    break
+
                 if isinstance(config["parent"], str):
                     if len(config["subDevices"]) == 0 and config["deviceName"] != "XLS200":
                             # If there are no subdevices and it's not an "empty" Multiboxdevice
@@ -405,6 +421,9 @@ class _GetValuesThread(threading.Thread):
 
                     else:
                         for subID, subInput in config["subDevices"].iteritems():
+                            if self.stop_event.is_set():
+                                break
+
                             rv = self.devices[key].getRawValue(input = subInput)
                             if rv:
                                 self.rawValues[subID] = rv
