@@ -21,6 +21,8 @@
 
 import si
 
+deviceClassNames = ("TecpelDMM8061", "VoltcraftVC820", "XLS200", "KernPCB", "BS600")
+
 class Device(object):
     """The base device class.
 
@@ -37,7 +39,7 @@ class Device(object):
     _dtr = False # data terminal ready rs232 flag (used for setting up the rs232 connection)
     _ownSer = False # does the device own the serial connection or is it just a user
     # (e.g. one of three devices, which are behind a multibox)
-    _timeout = 0.1 # timeout value for reading from serial device
+    _timeout = 0.2 # timeout value for reading from serial device
 
     def __init__(self, ser):
         """Setup a device as new device or as subdevice.
@@ -253,11 +255,12 @@ class NullValue(Value):
 
 
 class TecpelDMM8061(Device):
-    """This class represents a TECPEL DMM 8061 or VOLTCRAFT VC 840 Multimeter."""
+    """This class represents a TECPEL DMM 8061 multimeter."""
     
     _baudrate = 2400
     _rts = False
     _dtr = True
+    _timeout = 0.5
 
     class __Value(Value):
         class __Digit:
@@ -519,6 +522,14 @@ class TecpelDMM8061(Device):
         return None
 
 
+class VoltcraftVC820(TecpelDMM8061):
+    """This class represents a VOLTCRAFT VC 840 multimeter
+
+    The VC 40 is identical to the TecpelDMM8061.
+
+    """
+
+
 class XLS200(MultiboxDevice):
     """This class represents the XLS200 MultiboxDevice.
 
@@ -631,6 +642,9 @@ class XLS200(MultiboxDevice):
         elif input == 3:
             self._changeInput(3)
             return self._in3.getRawValue()
+        
+        else:
+            return None
 
     def getDevice(self, input):
         if input == 1:
