@@ -115,10 +115,10 @@ class DisplayWidget(QtGui.QWidget):
     twoValueCalibration = (0.0, 0.0), (1.0, 1.0)
     slopeInterceptCalibration = 1.0, 0.0
 
-    is1PrefixIndex = 2 # QComboBox Index
-    should1PrefixIndex = 2 # QComboBox Index
-    is2PrefixIndex = 2 # QComboBox Index
-    should2PrefixIndex = 2 # QComboBox Index
+    is1PrefixIndex = 0 # QComboBox Index
+    should1PrefixIndex = 0 # QComboBox Index
+    is2PrefixIndex = 0 # QComboBox Index
+    should2PrefixIndex = 0 # QComboBox Index
 
     is1 = 0
     should1 = 0
@@ -184,12 +184,8 @@ class DisplayWidget(QtGui.QWidget):
 
         popup.exec_()
 
-        self.twoValueCalibration = ((popup.is1.value() * si.getFactor(str(popup.is1Prefix.currentText())),
-                popup.should1.value()  * si.getFactor(str(popup.should1Prefix.currentText()))),
-            (popup.is2.value() * si.getFactor(str(popup.is2Prefix.currentText())), 
-                popup.should2.value() * si.getFactor(str(popup.should2Prefix.currentText()))))
-
-        self.slopeInterceptCalibration = popup.slope.value(), popup.intercept.value()
+        self.twoValueCalibration = popup.twoValueCalibration
+        self.slopeInterceptCalibration = popup.slopeInterceptCalibration
 
         self.is1 = popup.is1.value()
         self.should1 = popup.should1.value()
@@ -246,8 +242,103 @@ class DeviceSettingsDialog(QtGui.QDialog):
         self.deviceID = deviceID
         self.dm = dm
 
+        self.settings = QtCore.QSettings()
+
         self.get1.clicked.connect(self.setCurrentValue1)
         self.get2.clicked.connect(self.setCurrentValue2)
+        self.saveButton.clicked.connect(self.save)
+        self.loadButton.clicked.connect(self.load)
+
+        self.timer = QtCore.QTimer()
+        self.timer.timeout.connect(self.update)
+        self.timer.start(50)
+
+    def update(self):
+        self.twoValueCalibration = ((self.is1.value() * si.getFactor(str(self.is1Prefix.currentText())),
+                self.should1.value()  * si.getFactor(str(self.should1Prefix.currentText()))),
+            (self.is2.value() * si.getFactor(str(self.is2Prefix.currentText())), 
+                self.should2.value() * si.getFactor(str(self.should2Prefix.currentText()))))
+
+        self.slopeInterceptCalibration = self.slope.value(), self.intercept.value()
+        
+        if self.slopeInterceptButton.isChecked():
+            self = slopeInterceptCalibration
+        elif self.valuesButton.isChecked():
+            self = twoValueCalibration
+
+
+        crv = self.dm.getCalibratedLastRawValue(self.deviceID, self.calibration, self.unit)
+        self.calibratedLabel.setText(str(crv.getDisplayedValue() * crv.getFactor()))
+
+        rv = self.dm.getLastRawValue(self.deviceID)
+        self.normalLabel.setText(str(rv.getDisplayedValue() * rv.getFactor()))
+
+
+    def save(self):
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "is1", 
+            self.is1.value())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "should1", 
+            self.should1.value())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "is2", 
+            self.is2.value())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "should2", 
+            self.should2.value())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "is1PrefixIndex", 
+            self.is1Prefix.currentIndex())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "should1PrefixIndex", 
+            self.should1Prefix.currentIndex())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "is2PrefixIndex", 
+            self.is2Prefix.currentIndex())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "should2PrefixIndex", 
+            self.should2Prefix.currentIndex())
+
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "slope",
+            self.slope.value())
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "intercept",
+            self.intercept.value())
+
+        self.settings.setValue("calibration/" +
+            self.slotComboBox.currentText() + "unit",
+            self.unit.text())
+
+    def load(self):
+        self.is1.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "is1", 0).toInt()[0])
+        self.should1.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "should1", 1).toInt()[0])
+        self.is2.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "is2", 1).toInt()[0])
+        self.should2.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "should2", 1).toInt()[0])
+        self.is1Prefix.setCurrentIndex(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "is1PrefixIndex", "").toInt()[0])
+        self.should1Prefix.setCurrentIndex(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "should1PrefixIndex", "").toInt()[0])
+        self.is2Prefix.setCurrentIndex(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "is2PrefixIndex", "").toInt()[0])
+        self.should2Prefix.setCurrentIndex(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "should2PrefixIndex", "").toInt()[0])
+
+        self.slope.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "slope", 1).toInt()[0])
+        self.intercept.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "intercept", 0).toInt()[0])
+
+        self.intercept.setValue(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "unit", 0).toInt()[0])
+
+        self.unit.setText(self.settings.value("calibration/" +
+            self.slotComboBox.currentText() + "unit", "").toString())
 
     def setCurrentValue1(self):
         """Get current displayedValue from Device."""
@@ -309,7 +400,6 @@ class MainWindow(QtGui.QMainWindow):
         self.timer.timeout.connect(self.update)
         self.timer.start(50)
 
-        self.settings = QtCore.QSettings()
         self.officePath = str(self.settings.value("office/path", "").toString())
 
         self.loggingInterval.setValue(self.settings.value("logging/interval", 1).toInt()[0])
@@ -353,35 +443,39 @@ class MainWindow(QtGui.QMainWindow):
 
             if device == "XLS200":
                 xls200Popup = Xls200Dialog()
+                
+                validDevices = self.dm.getValidDevices()
+                validDevices.remove("XLS200") # Chaining XLS200 devices is not supported
+
                 for i in (xls200Popup.subdevice1ComboBox,
                         xls200Popup.subdevice2ComboBox, xls200Popup.subdevice3ComboBox):
-                    i.addItems([""] + self.dm.getValidDevices())
+                    i.addItems([self.tr("No device")] + validDevices)
 
                 xls200Popup.exec_()
 
                 if xls200Popup.result():
                     xls200ID = self.dm.openDevice(device, port)
-                    sub1 = str(xls200Popup.subdevice1ComboBox.currentText())
-                    sub2 = str(xls200Popup.subdevice2ComboBox.currentText())
-                    sub3 = str(xls200Popup.subdevice3ComboBox.currentText())
                     
-                    if sub1 != "":
-                            deviceID = self.dm.openSubdevice(sub1, xls200ID, 1)
-                            sub1Widget = DisplayWidget(deviceID, self.dm)
-                            self.verticalLayout.addWidget(sub1Widget)
-                            self.displayWidgets[deviceID] = sub1Widget
+                    if xls200Popup.subdevice1ComboBox.currentIndex() != 0:
+                        sub1 = unicode(xls200Popup.subdevice1ComboBox.currentText())
+                        deviceID = self.dm.openSubdevice(sub1, xls200ID, 1)
+                        sub1Widget = DisplayWidget(deviceID, self.dm)
+                        self.verticalLayout.addWidget(sub1Widget)
+                        self.displayWidgets[deviceID] = sub1Widget
 
-                    if sub2 != "":
-                            deviceID = self.dm.openSubdevice(sub2, xls200ID, 2)
-                            sub2Widget = DisplayWidget(deviceID, self.dm)
-                            self.verticalLayout.addWidget(sub2Widget)
-                            self.displayWidgets[deviceID] = sub2Widget
+                    if xls200Popup.subdevice2ComboBox.currentIndex() != 0:
+                        sub2 = unicode(xls200Popup.subdevice2ComboBox.currentText())
+                        deviceID = self.dm.openSubdevice(sub2, xls200ID, 2)
+                        sub2Widget = DisplayWidget(deviceID, self.dm)
+                        self.verticalLayout.addWidget(sub2Widget)
+                        self.displayWidgets[deviceID] = sub2Widget
 
-                    if sub3 != "":
-                            deviceID = self.dm.openSubdevice(sub3, xls200ID, 3)
-                            sub3Widget = DisplayWidget(deviceID, self.dm)
-                            self.verticalLayout.addWidget(sub3Widget)
-                            self.displayWidgets[deviceID] = sub3Widget
+                    if xls200Popup.subdevice3ComboBox.currentIndex() != 0:
+                        sub3 = unicode(xls200Popup.subdevice3ComboBox.currentText())
+                        deviceID = self.dm.openSubdevice(sub3, xls200ID, 3)
+                        sub3Widget = DisplayWidget(deviceID, self.dm)
+                        self.verticalLayout.addWidget(sub3Widget)
+                        self.displayWidgets[deviceID] = sub3Widget
 
             else:
                 deviceID = self.dm.openDevice(device, port)
@@ -433,16 +527,16 @@ class MainWindow(QtGui.QMainWindow):
         import os
 
         if self.tmpfile:
-	        popup = QtGui.QFileDialog()
-	        filename = QtGui.QFileDialog.getSaveFileName(self, self.tr("Save file"), os.path.expanduser("~"), "")
+            popup = QtGui.QFileDialog()
+            filename = QtGui.QFileDialog.getSaveFileName(self, self.tr("Save file"),
+                os.path.expanduser("~/" + str(self.tr("log", "Default filename of the exported log")) + ".csv"),
+                "CSV (*.csv)")
 
-	        if filename[-4:] != ".csv":
-	            filename += ".csv"
-
-	        self.tmpfile.seek(0, 0)
-	        with open(filename, 'w') as stream:
-	            stream.write(self.tmpfile.read())
-	        self.pathToLogFile = str(filename)
+            if filename != "":
+                self.tmpfile.seek(0, 0)
+                with open(filename, 'w') as stream:
+                    stream.write(self.tmpfile.read())
+                self.pathToLogFile = str(filename)
         else:
             popup = DoReallyDialog(self.tr("Warning"),
                 self.tr("You first have to log something, to save it."))
